import random
import os
import shutil
import datetime

def randmatch():
    lst = [chr(a) for a in range(33, 126)]
    random.shuffle(lst)
    return dict(zip(lst, lst[::-1]))

def backup_rotors(file_name):
    home_dir = os.path.expanduser("~")
    backup_folder = os.path.join(home_dir, ".enigmabackups")
    original_file_path = file_name
    backup_file_path = os.path.join(backup_folder, os.path.basename(file_name))

    if not os.path.exists(backup_folder):
        os.makedirs(backup_folder)

    if os.path.exists(original_file_path):
        backup_path = f"{backup_file_path}_{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}.backup"
        shutil.copy(original_file_path, backup_path)
        print(f"Backup of existing rotors saved as: {backup_path}")

def create_rotors(count):
    # Get the directory of the current script (mshuffle.py)
    dir_path = os.path.dirname(os.path.realpath(__file__))

    # Construct the file path for defaultRotor.py in the same directory
    file_path = os.path.join(dir_path, 'defaultRotor.py')

    rotors = [randmatch() for _ in range(count)]
    with open(file_path, 'w') as f:
        f.write('ROTORS = ')
        f.write(repr(rotors))
        f.write('\n')

    print(f"New rotors created and saved in: {file_path}")

if __name__ == "__main__":
    rotor_file = 'defaultRotor.py'
    count = int(input('How many rotors do you want to create? '))
    create_rotors(count)
    # create_rotors(rotor_file, count)



import json
import argparse
from .engine import *
import getpass
from .mshuffle import *
from enigmacifra import __version__
from .defaultRotor import ROTORS

def main():
    parser = argparse.ArgumentParser(description="Enigma-like encryption and decryption tool")
    parser.add_argument('--version', action='version', version=f'Enigma CLI Version: {__version__}', help="Show program's version number and exit")
    subparsers = parser.add_subparsers(dest='command')

    # Add subparsers for the shuffle, encrypt, and decrypt commands
    parser_shuffle = subparsers.add_parser('get-new-rotors', help='Shuffle and create new rotors')
    parser_encrypt = subparsers.add_parser('encrypt', help='Encrypt a message')
    parser_decrypt = subparsers.add_parser('decrypt', help='Decrypt a message')
    args = parser.parse_args()


    if args.command == "get-new-rotors":
        red_start = "\033[91m"
        red_end = "\033[0m"

        # First confirmation
        warning_message = (
            red_start +
            "WARNING: Proceeding will overwrite existing rotor configurations. "
            "This will make previously encrypted data undecipherable unless you "
            "have a backup of the current rotors.\nAre you sure you want to continue? [Y/N]: " +
            red_end
        )
        first_confirm = input(warning_message)

        if first_confirm.lower() == 'y':
            # Second confirmation
            second_confirm = input("Please type 'yes' to confirm: ")
            if second_confirm.lower() == 'yes':
                rotor_file = "defaultRotor.py"
                backup_rotors(rotor_file)
                count = int(input("How many rotors do you want to create? "))
                create_rotors(count)
            else:
                print("Operation cancelled.")
        else:
            print("Operation cancelled.")


    elif args.command in ["encrypt", "decrypt"]:
        # Load the rotor list
        Rotor_lst = ROTORS
        # Prompt for PIN
        pin = getpass.getpass("Enter 8-digit PIN to encrypt/decrypt the password: ")
        pin_confirm = getpass.getpass("Please confirm your PIN: ")
        max_wrong_counts = 1
        while pin_confirm != pin:
            max_wrong_counts += 1
            if max_wrong_counts == 3:
                print("Maximum attempts reached. Exiting.")
                return
            print("----PINs didn't match----")
            pin = getpass.getpass("Enter 8-digit PIN to encrypt/decrypt the password: ")
            pin_confirm = getpass.getpass("Please confirm your PIN: ")
            
        if len(pin) != 8 or not pin.isnumeric():
            print("Invalid PIN. Please enter a 8-digit numeric PIN.")
            return

        PIN = [int(i) for i in pin]
        PEL = Encrypting_package(PIN, Rotor_lst)

        if args.command == "encrypt":
            password = input("Enter the message to be encrypted: ")
            encrypted_message = process_e(PEL, password)
            print("Your encrypted message is: ", encrypted_message)

        elif args.command == "decrypt":
            encrypted_message = input("Enter the encrypted message: ")
            decrypted_message = process_d(PEL, encrypted_message)
            print("Your decrypted message: ", decrypted_message)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()



import json

def rotate_single_dict(x, dc):
    """Rotates the given dictionary 'dc' by 'x' positions."""
    dc_rotated = {}
    keys, values = zip(*dc.items())
    for cnt in range(len(dc)):
        dc_rotated[keys[cnt]] = values[(cnt - x) % len(dc)]
    return dc_rotated


def Encrypting_package(pin, rotor_list):
    """Encrypt a package using a PIN and a list of rotors."""
    package_encrypted = []
    for i in range(6):
        pin_digit = pin[i]
        package_encrypted.append(rotate_single_dict(pin_digit, rotor_list[i]))
    return package_encrypted


def passing(pin_encrypted_list, char):
    """Pass a character through the encrypted PIN list to transform it."""
    for rotor in pin_encrypted_list:
        char = rotor[char]
    return char


def get_key_from_value(dc, value):
    """Find the key in a dictionary corresponding to a given value."""
    for key, val in dc.items():
        if val == value:
            return key
    raise ValueError("Value not found in dictionary")


def unpassing(pin_encrypted_list, char):
    """Reverse the transformation process of a character using the encrypted PIN list."""
    for i in range(1, 7):
        char = get_key_from_value(pin_encrypted_list[-i], char)
    return char


"""
Tattoo translates to 'flfvv'. To prevent this repetition, we will update the 'PEL' (Pin Encrypted List) or 'packageE' after every use. This update involves rotating the list by 1 position for each encryption process.
"""


def updating(pin_encrypted_list, encrypt=True):
    """
    Update the PIN Encrypted List (PEL) for encryption or decryption.
    By default, it is set for encryption."""
    direction = 1 if encrypt else -1
    pin = [direction] * 6
    return Encrypting_package(pin, pin_encrypted_list)


def process_e(pin_encrypted_list, password):
    """
    Encrypt a password using the PIN Encrypted List (PEL)."""
    encrypted_password = ""
    for char in password:
        encrypted_password += passing(pin_encrypted_list, char)
        pin_encrypted_list = updating(pin_encrypted_list)
    return encrypted_password


def process_d(PEL, passwordE):
    password = ""
    for str_ in passwordE:
        password += unpassing(PEL, str_)
        PEL = updating(PEL)
    return password


"""The above rotor list is to be well backed-up and should never be disturbed as it has contains the shuffled rotors"""
if __name__ == "__main__":
    # Now, We'll ask for the 6-digit PIN

    pin = input("Enter your 6-digit PIN to encrypt/decrypt the password: ")
    if len(pin) == 8 and pin.isnumeric():
        PIN = [int(i) for i in pin]
        PEL = Encrypting_package(PIN, Rotor_lst)
        eVSd = input('Please enter "E" to encrypt or "D" to decrypt: ')
        if eVSd.lower() == "e":
            password = input("Enter the password to be encrypted: ")
            passwordE = process_e(PEL, password)
            print("Your encrypted password is: ", passwordE)
        elif eVSd.lower() == "d":
            passwordE = input("Enter your encrypted passowrd: ")
            password = process_d(PEL, passwordE)
            print("Your decrypted password: ", password)
        else:
            print('Get lost!! I aksed to enter only "E" or "D"')
            print("Thank you for using me!")
    else:
        if pin.isnumeric() == False:
            print("The above is not PIN, Please only enter numbers")
        else:
            if len(pin) == 1:
                print(f"The above PIN is of 1-digit only")
            elif len(pin) < 6:
                print(f"The above PIN is of only {len(pin)}-digits")
            else:
                print(f"The above PIN is of {len(pin)}-digits")
        print("Program ended!")



from setuptools import setup, find_packages
from enigmacifra import __version__

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='enigmacifra',  
    version=__version__,
    author="Darshan P.",
    author_email="drshnp@outlook.com",
    license="MIT",
    description="PIN encrypted enigma machine cipher texts",
    long_description=long_description,
    url="https://github.com/1darshanpatil/enigmacipher", 
    packages=find_packages(),
    package_data={
        'enigmacifra': ['defaultRotor.txt'],
    },
    entry_points={
        "console_scripts": [
            "enigma=enigmacifra.cliengine:main",
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    include_package_data=True,
    keywords="enigma, cipher, encryption",
    project_urls={
        "Documentation": "https://github.com/1darshanpatil/enigmacipher#readme",
        "Source": "https://github.com/1darshanpatil/enigmacipher",
        "Tracker": "https://github.com/1darshanpatil/enigmacipher/issues",
    },
)
